#ifndef PPSSUMM_FITSCOLUMN
#define PPSSUMM_FITSCOLUMN

/* 
 *
 * Functions for reading FIT columns via DAL
 * into an iterator via a template function
 * Rows to read are provided by an iterator sequence
 * of unsigned integers
 *
 * No conversion is done here so the output has to be fo the right type!
 *
 */

#include <string>
#include <iostream>
#include "string_algo.h"

#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

template< typename Oi_ , typename Ii_ >
Oi_ bool8Column( const Column* col, Oi_ result_ ,  Ii_ first_ , Ii_ last_ ) 
{
    ColumnData *coldat = col->data();
    bool8* data = coldat->boolData();
    while (first_ != last_) {
        
        if ( (*first_) < col->parent()->rows() ) {
            *result_++ = data[*first_] ;
        } else {
            errstr << "bool8Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
    }
    delete (coldat);
    return result_;
}

template< typename Oi_ , typename Ii_ >
Oi_ int8Column( const Column* col , Oi_ result_ , Ii_ first_ , Ii_ last_ )
{
    ColumnData *coldat = col->data();
    int8* data = coldat->int8Data();
    while (first_ != last_) {
        
        if ( static_cast<unsigned int>(*first_) < col->parent()->rows()) {
            *result_++ =  data[*first_] ;
        } else {
            errstr << "int8Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
    }
    delete (coldat);
    return result_;
}

template< typename Oi_ , typename Ii_ >
Oi_ uint16Column( const Column* col, Oi_ result_ , Ii_ first_, Ii_ last_ ) 
{
    ColumnData *coldat = col->data();
    uint16* data = coldat->uint16Data();
    while (first_ != last_) {
        
        if ( static_cast<unsigned int>(*first_) < col->parent()->rows()) {
            *result_++ =  data[*first_] ;
        } else {
            errstr << "uint16Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
    }
    delete (coldat);
    return result_;
}


template< typename Oi_ , typename Ii_ >
Oi_ int16Column( const Column* col , Oi_ result_ , Ii_ first_ , Ii_ last_ ) 
{
    ColumnData *coldat = col->data();
    int16* data = coldat->int16Data();
    while (first_ != last_) {
        
        if ( static_cast<unsigned int>(*first_) < col->parent()->rows()) {
            *result_++ =  data[*first_] ;
        } else {
            errstr << "int16Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
    }
    delete (coldat);
    return result_;
}

template< typename Oi_ , typename Ii_ >
Oi_ uint32Column( const Column* col , Oi_ result_ , Ii_ first_ , Ii_ last_ ) 
{
    ColumnData *coldat = col->data();
    uint32* data = coldat->uint32Data();
    while (first_ != last_) {
        
        if ( static_cast<unsigned int>(*first_) < col->parent()->rows()) {
            *result_++ =  data[*first_] ;
        } else {
            errstr << "uint32Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
    }
    delete (coldat);
    return result_;
}


template< typename Oi_ , typename Ii_ >
Oi_ int32Column( const Column* col, Oi_ result_ , Ii_ first_ , Ii_ last_ ) 
{
    ColumnData *coldat = col->data();
    int32* data = coldat->int32Data();
    while (first_ != last_) {
        
        if (static_cast<unsigned int>(*first_) < col->parent()->rows()) {
            *result_++ =  data[*first_] ;
        } else {
            errstr << "int32Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
    }
    delete (coldat);
    return result_;
}

template< typename Oi_ , typename Ii_ >
Oi_ real32Column( const Column* col, Oi_ result_ ,Ii_ first_, Ii_ last_ ) 
{
    ColumnData *coldat = col->data();
    real32* data = coldat->real32Data();
    while (first_ != last_) {
        
        if ( static_cast<unsigned int>(*first_) < col->parent()->rows()) {
            *result_++ =  data[*first_] ;
        } else {
            errstr << "real32Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
    }
    delete (coldat);
    return result_;
}

template< typename Oi_ , typename Ii_ >
Oi_ real64Column( const Column* col, Oi_ result_ , Ii_ first_ , Ii_ last_ ) 
{
    ColumnData *coldat = col->data();
    real64* data = coldat->real64Data();
    while (first_ != last_) {
        
        if ( static_cast<unsigned int>(*first_) < col->parent()->rows()) {
                
            *result_++ =  data[*first_] ;
        } else {
            errstr << "real64Column: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
    }
    delete (coldat);
    return result_;
}

template< typename Oi_ , typename Ii_ >
Oi_ stringColumn( const Column* col , Oi_ result_ , Ii_ first_ , Ii_ last_ ) 
{
    unsigned int width = col->elements();

    ColumnData *coldat = col->data();
    char* data = coldat->stringData();
    // tostring strips trailing \0 left on strings by this extraction process
    ppssumm_internal::toString<std::string> tostring;
    while (first_ != last_) {

        if (static_cast<unsigned int>(*first_) < col->parent()->rows()) {

            std::string s = std::string(data + (*first_)*width , data + (*first_+1)*width) ;
            *result_++ = tostring( s );
        } else { 
            errstr << "stringColumn: Attempt to read beyond end of column." << Sas::fatal("ColumnAccessOutofBounds"); 
        }
        first_++;
        
    }
    delete (coldat);
    return result_;
}



#endif
